@@ -60,6 +60,7 @@ etcs_init (ETableColumnSpecification *specification)
 	
 	specification->cell          = NULL;
 	specification->compare       = NULL;
+	specification->priority      = 0;
 }
 
 E_MAKE_TYPE(e_table_column_specification, "ETableColumnSpecification", ETableColumnSpecification, etcs_class_init, etcs_init, PARENT_TYPE);
@@ -88,6 +89,7 @@ e_table_column_specification_load_from_node (ETableColumnSpecification *etcs,
 
 	etcs->cell          = e_xml_get_string_prop_by_name(node, "cell");
 	etcs->compare       = e_xml_get_string_prop_by_name(node, "compare");
+	etcs->priority      = e_xml_get_integer_prop_by_name_with_default (node, "priority", 0);
 
 	if (etcs->title == NULL)
 		etcs->title = g_strdup("");
@@ -113,6 +115,8 @@ e_table_column_specification_save_to_node (ETableColumnSpecification *specificat
 
 	e_xml_set_string_prop_by_name(node, "cell", specification->cell);
 	e_xml_set_string_prop_by_name(node, "compare", specification->compare);
+	if (specification->priority != 0)
+		e_xml_set_integer_prop_by_name (node, "priority", specification->priority);
 
 	return node;
 }
