@@ -67,6 +67,7 @@ enum {
 	ARG_CURSOR_MODE,
 	ARG_LENGTH_THRESHOLD,
 	ARG_CURSOR_ROW,
+	ARG_UNIFORM_ROW_HEIGHT,
 	
 	ARG_MINIMUM_WIDTH,
 	ARG_WIDTH,
@@ -83,12 +84,10 @@ static void e_table_item_focus (ETableItem *eti, int col, int row, GdkModifierTy
 static void eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti);
 static void eti_cursor_activated (ESelectionModel *selection, int row, int col, ETableItem *eti);
 static void eti_selection_change (ESelectionModel *selection, ETableItem *eti);
-#ifdef SHOW_ON_CHANGED
-static void eti_request_region_show (ETableItem *eti,
-				     int start_col, int start_row,
-				     int end_col, int end_row);
-#endif
-#define ETI_ROW_HEIGHT(eti,row) ((eti)->height_cache && (eti)->height_cache[(row)] != -1 ? (eti)->height_cache[(row)] : eti_row_height((eti),(row)))
+
+#define ETI_SINGLE_ROW_HEIGHT(eti) ((eti)->uniform_row_height_cache != -1 ? (eti)->uniform_row_height_cache : eti_row_height((eti), -1))
+#define ETI_MULTIPLE_ROW_HEIGHT(eti,row) ((eti)->height_cache && (eti)->height_cache[(row)] != -1 ? (eti)->height_cache[(row)] : eti_row_height((eti),(row)))
+#define ETI_ROW_HEIGHT(eti,row) ((eti)->uniform_row_height ? ETI_SINGLE_ROW_HEIGHT ((eti)) : ETI_MULTIPLE_ROW_HEIGHT((eti),(row)))
 
 inline static gint
 model_to_view_row(ETableItem *eti, int row)
@@ -501,7 +500,7 @@ confirm_height_cache (ETableItem *eti)
 {
 	int i;
 
-	if (eti->height_cache)
+	if (eti->uniform_row_height || eti->height_cache)
 		return;
 	eti->height_cache = g_new(int, eti->rows);
 	for (i = 0; i < eti->rows; i++) {
@@ -538,8 +537,14 @@ free_height_cache (ETableItem *eti)
 		g_free (eti->height_cache);
 	eti->height_cache = NULL;
 	eti->height_cache_idle_count = 0;
-	
-	if (eti->height_cache_idle_id == 0)
+	eti->uniform_row_height_cache = -1;
+
+	if (eti->uniform_row_height && eti->height_cache_idle_id != 0) {
+		g_source_remove(eti->height_cache_idle_id);
+		eti->height_cache_idle_id = 0;
+	}
+
+	if ((!eti->uniform_row_height) && eti->height_cache_idle_id == 0)
 		eti->height_cache_idle_id = g_idle_add_full(G_PRIORITY_LOW, (GSourceFunc) height_cache_idle, eti, NULL);
 }
 
@@ -560,20 +565,25 @@ calculate_height_cache (ETableItem *eti)
 static int
 eti_row_height (ETableItem *eti, int row)
 {
-	if (!eti->height_cache) {
-		calculate_height_cache (eti);
-	}
-	if (eti->height_cache[row] == -1) {
-		eti->height_cache[row] = eti_row_height_real(eti, row);
-		if (row > 0 && 
-		    eti->length_threshold != -1 && 
-		    eti->rows > eti->length_threshold &&
-		    eti->height_cache[row] != eti_row_height(eti, 0)) {
-			eti->needs_compute_height = 1;
-			e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(eti));
+	if (eti->uniform_row_height) {
+		eti->uniform_row_height_cache = eti_row_height_real (eti, -1);
+		return eti->uniform_row_height_cache;
+	} else {
+		if (!eti->height_cache) {
+			calculate_height_cache (eti);
 		}
+		if (eti->height_cache[row] == -1) {
+			eti->height_cache[row] = eti_row_height_real(eti, row);
+			if (row > 0 && 
+			    eti->length_threshold != -1 && 
+			    eti->rows > eti->length_threshold &&
+			    eti->height_cache[row] != eti_row_height(eti, 0)) {
+				eti->needs_compute_height = 1;
+				e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(eti));
+			}
+		}
+		return eti->height_cache[row];
 	}
-	return eti->height_cache[row];
 }
 
 /*
@@ -592,41 +602,46 @@ static int
 eti_get_height (ETableItem *eti)
 {
 	const int rows = eti->rows;
-	int row;
-	int height;
 	int height_extra = eti->horizontal_draw_grid ? 1 : 0;
 
 	if (rows == 0)
 		return 0;
 
-	if (eti->length_threshold != -1){
-		if (rows > eti->length_threshold){
-			int row_height = eti_row_height(eti, 0);
-			if (eti->height_cache) {
-				height = 0;
-				for (row = 0; row < rows; row++) {
-					if (eti->height_cache[row] == -1) {
-						height += (row_height + height_extra) * (rows - row);
-						break;
+	if (eti->uniform_row_height) {
+		int row_height = eti_row_height(eti, -1);
+		return ((row_height + height_extra) * rows + height_extra);
+	} else {
+		int height;
+		int row;
+		if (eti->length_threshold != -1){
+			if (rows > eti->length_threshold){
+				int row_height = eti_row_height(eti, 0);
+				if (eti->height_cache) {
+					height = 0;
+					for (row = 0; row < rows; row++) {
+						if (eti->height_cache[row] == -1) {
+							height += (row_height + height_extra) * (rows - row);
+							break;
+						}
+						else
+							height += eti->height_cache[row] + height_extra;
 					}
-					else
-						height += eti->height_cache[row] + height_extra;
-				}
-			} else
-				height = (eti_row_height (eti, 0) + height_extra) * rows;
+				} else
+					height = (eti_row_height (eti, 0) + height_extra) * rows;
 
-			/*
-			 * 1 pixel at the top
-			 */
-			return height + height_extra;
+				/*
+				 * 1 pixel at the top
+				 */
+				return height + height_extra;
+			}
 		}
+
+		height = height_extra;
+		for (row = 0; row < rows; row++)
+			height += eti_row_height (eti, row) + height_extra;
+
+		return height;
 	}
-
-	height = height_extra;
-	for (row = 0; row < rows; row++)
-		height += eti_row_height (eti, row) + height_extra;
-
-	return height;
 }
 
 static void
@@ -647,63 +662,52 @@ eti_item_region_redraw (ETableItem *eti, int x0, int y0, int x1, int y1)
 	gnome_canvas_request_redraw (item->canvas, rect.x0, rect.y0, rect.x1, rect.y1);
 }
 
-/*
- * Callback routine: invoked before the ETableModel has suffers a change
- */
-static void
-eti_table_model_pre_change (ETableModel *table_model, ETableItem *eti)
-{
-	if (eti_editing (eti))
-		e_table_item_leave_edit (eti);
-}
-
-/*
- * Callback routine: invoked when the ETableModel has suffered a change
- */
-static void
-eti_table_model_changed (ETableModel *table_model, ETableItem *eti)
-{
-	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
-		return;
-#ifdef SHOW_ON_CHANGED
-	int view_row;
-#endif
-
-	eti->rows = e_table_model_row_count (eti->table_model);
-
-	free_height_cache(eti);
-
-	eti->needs_compute_height = 1;
-	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (eti));
-	eti->needs_redraw = 1;
-	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (eti));
-
-#ifdef SHOW_ON_CHANGED
-	view_row = model_to_view_row(eti, eti->cursor_row);
-	if (view_row >= 0 && eti->cursor_col >= 0)
-		eti_request_region_show (eti, eti->cursor_col, view_row, eti->cursor_col, view_row);
-#endif
-}
-
 /*
  * Computes the distance between @start_row and @end_row in pixels
  */
 int
 e_table_item_row_diff (ETableItem *eti, int start_row, int end_row)
 {
-	int row, total;
 	int height_extra = eti->horizontal_draw_grid ? 1 : 0;
 
-	total = 0;
 	if (start_row < 0)
 		start_row = 0;
 	if (end_row > eti->rows)
 		end_row = eti->rows;
 
-	for (row = start_row; row < end_row; row++)
-		total += eti_row_height (eti, row) + height_extra;
+	if (eti->uniform_row_height) {
+		return ((end_row - start_row) * (eti_row_height(eti, -1) + height_extra));
+	} else {
+		int row, total;
+		total = 0;
+		for (row = start_row; row < end_row; row++)
+			total += eti_row_height (eti, row) + height_extra;
 
-	return total;
+		return total;
+	}
+}
+
+static void
+eti_get_region (ETableItem *eti,
+		int start_col, int start_row,
+		int end_col, int end_row,
+		int *x1p, int *y1p,
+		int *x2p, int *y2p)
+{
+	int x1, y1, x2, y2;
+	
+	x1 = e_table_header_col_diff (eti->header, 0, start_col);
+	y1 = e_table_item_row_diff (eti, 0, start_row);
+	x2 = x1 + e_table_header_col_diff (eti->header, start_col, end_col + 1);
+	y2 = y1 + e_table_item_row_diff (eti, start_row, end_row + 1);
+	if (x1p)
+		*x1p = x1;
+	if (y1p)
+		*y1p = y1;
+	if (x2p)
+		*x2p = x2;
+	if (y2p)
+		*y2p = y2;
 }
 
 /*
@@ -721,19 +725,19 @@ eti_request_region_redraw (ETableItem *eti,
 			   int start_col, int start_row,
 			   int end_col, int end_row, int border)
 {
-	int x1, y1, width, height;
+	int x1, y1, x2, y2;
 
 	if (eti->rows > 0) {
-		
-		x1 = e_table_header_col_diff (eti->header, 0, start_col);
-		y1 = e_table_item_row_diff (eti, 0, start_row);
-		width = e_table_header_col_diff (eti->header, start_col, end_col + 1);
-		height = e_table_item_row_diff (eti, start_row, end_row + 1);
+
+		eti_get_region (eti,
+				start_col, start_row,
+				end_col, end_row,
+				&x1, &y1, &x2, &y2);
 		
 		eti_item_region_redraw (eti, eti->x1 + x1 - border,
 					eti->y1 + y1 - border,
-					eti->x1 + x1 + width + 1 + border,
-					eti->y1 + y1 + height + 1 + border);
+					eti->x1 + x2 + 1 + border,
+					eti->y1 + y2 + 1 + border);
 	}
 }
 
@@ -749,11 +753,11 @@ eti_request_region_show (ETableItem *eti,
 			 int end_col, int end_row, int delay)
 {
 	int x1, y1, x2, y2;
-	
-	x1 = e_table_header_col_diff (eti->header, 0, start_col);
-	y1 = e_table_item_row_diff (eti, 0, start_row);
-	x2 = x1 + e_table_header_col_diff (eti->header, start_col, end_col + 1);
-	y2 = y1 + e_table_item_row_diff (eti, start_row, end_row + 1);
+
+	eti_get_region (eti,
+			start_col, start_row,
+			end_col, end_row,
+			&x1, &y1, &x2, &y2);
 
 	if (delay)
 		e_canvas_item_show_area_delayed(GNOME_CANVAS_ITEM(eti), x1, y1, x2, y2, delay);
@@ -761,14 +765,149 @@ eti_request_region_show (ETableItem *eti,
 		e_canvas_item_show_area(GNOME_CANVAS_ITEM(eti), x1, y1, x2, y2);
 }
 
+static void
+eti_show_cursor (ETableItem *eti, int delay)
+{
+	int cursor_row;
+	
+	if (!((GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED) && eti->cell_views_realized))
+		return;
+	
+	gtk_object_get(GTK_OBJECT(eti->selection),
+		       "cursor_row", &cursor_row,
+		       NULL);
+
+	if (cursor_row != -1) {
+		cursor_row = model_to_view_row (eti, cursor_row);
+		d(g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
+		eti_request_region_show (eti,
+					 0, cursor_row, eti->cols + 1, cursor_row + 1,
+					 delay);
+	}
+}
+
+static void
+eti_check_cursor_on_screen (ETableItem *eti)
+{
+	if (eti->cursor_x1 == -1 ||
+	    eti->cursor_y1 == -1 ||
+	    eti->cursor_x2 == -1 ||
+	    eti->cursor_y2 == -1)
+		return;
+
+	eti->cursor_on_screen = e_canvas_item_area_shown (GNOME_CANVAS_ITEM(eti),
+							  eti->cursor_x1,
+							  eti->cursor_y1,
+							  eti->cursor_x2,
+							  eti->cursor_y2);
+
+	d(g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+}
+
+static void
+eti_check_cursor_bounds (ETableItem *eti)
+{
+	int x1, y1, x2, y2;
+	int cursor_row;
+
+	if (!((GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED) && eti->cell_views_realized))
+		return;
+	
+	gtk_object_get(GTK_OBJECT(eti->selection),
+		       "cursor_row", &cursor_row,
+		       NULL);
+
+	if (cursor_row == -1) {
+		eti->cursor_x1 = -1;
+		eti->cursor_y1 = -1;
+		eti->cursor_x2 = -1;
+		eti->cursor_y2 = -1;
+		eti->cursor_on_screen = FALSE;
+		return;
+	}
+
+	cursor_row = model_to_view_row (eti, cursor_row);
+
+	d(g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
+
+	eti_get_region (eti,
+			0, cursor_row, eti->cols + 1, cursor_row + 1,
+			&x1, &y1, &x2, &y2);
+	eti->cursor_x1 = x1;
+	eti->cursor_y1 = y1;
+	eti->cursor_x2 = x2;
+	eti->cursor_y2 = y2;
+	eti->cursor_on_screen = e_canvas_item_area_shown (GNOME_CANVAS_ITEM(eti), x1, y1, x2, y2);
+
+	d(g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+}
+
+static void
+eti_maybe_show_cursor(ETableItem *eti, int delay)
+{
+	d(g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+	if (eti->cursor_on_screen)
+		eti_show_cursor (eti, delay);
+	eti_check_cursor_bounds (eti);
+}
+
+static gboolean
+eti_idle_maybe_show_cursor_cb (gpointer data)
+{
+	ETableItem *eti = data;
+	if (!GTK_OBJECT_DESTROYED (eti)) {
+		eti_maybe_show_cursor (eti, 0);
+	}
+	gtk_object_unref (GTK_OBJECT (eti));
+	return FALSE;
+}
+
+static void
+eti_idle_maybe_show_cursor(ETableItem *eti)
+{
+	gtk_object_ref (GTK_OBJECT (eti));
+	g_idle_add (eti_idle_maybe_show_cursor_cb, eti);
+}
+
+/*
+ * Callback routine: invoked before the ETableModel has suffers a change
+ */
+static void
+eti_table_model_pre_change (ETableModel *table_model, ETableItem *eti)
+{
+	eti_check_cursor_bounds (eti);
+	if (eti_editing (eti))
+		e_table_item_leave_edit (eti);
+}
+
+/*
+ * Callback routine: invoked when the ETableModel has suffered a change
+ */
+static void
+eti_table_model_changed (ETableModel *table_model, ETableItem *eti)
+{
+	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+		return;
+
+	eti->rows = e_table_model_row_count (eti->table_model);
+
+	free_height_cache(eti);
+
+	eti->needs_compute_height = 1;
+	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (eti));
+	eti->needs_redraw = 1;
+	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (eti));
+
+	eti_idle_maybe_show_cursor(eti);
+}
+
 static void
 eti_table_model_row_changed (ETableModel *table_model, int row, ETableItem *eti)
 {
 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
 		return;
-	if (eti->renderers_can_change_size &&
-	    eti->height_cache && eti->height_cache[row] != -1 &&
-	    eti_row_height_real(eti, row) != eti->height_cache[row]) {
+
+	if ((!eti->uniform_row_height) && eti->height_cache && eti->height_cache[row] != -1 && eti_row_height_real(eti, row) != eti->height_cache[row]) {
 		eti_table_model_changed (table_model, eti);
 		return;
 	}
@@ -781,9 +920,8 @@ eti_table_model_cell_changed (ETableModel *table_model, int col, int row, ETable
 {
 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
 		return;
-	if (eti->renderers_can_change_size &&
-	    eti->height_cache && eti->height_cache[row] != -1 &&
-	    eti_row_height_real(eti, row) != eti->height_cache[row]) {
+
+	if ((!eti->uniform_row_height) && eti->height_cache && eti->height_cache[row] != -1 && eti_row_height_real(eti, row) != eti->height_cache[row]) {
 		eti_table_model_changed (table_model, eti);
 		return;
 	}
@@ -806,6 +944,8 @@ eti_table_model_rows_inserted (ETableModel *table_model, int row, int count, ETa
 			eti->height_cache[i] = -1;
 	}
 
+	eti_idle_maybe_show_cursor(eti);
+
 	eti->needs_compute_height = 1;
 	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (eti));
 	eti->needs_redraw = 1;
@@ -828,6 +968,8 @@ eti_table_model_rows_deleted (ETableModel *table_model, int row, int count, ETab
 		memmove(eti->height_cache + row, eti->height_cache + row + count, (eti->rows - row) * sizeof(int));
 	}
 
+	eti_idle_maybe_show_cursor(eti);
+
 	eti->needs_compute_height = 1;
 	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (eti));
 	eti->needs_redraw = 1;
@@ -1026,8 +1168,10 @@ eti_destroy (GtkObject *object)
 	eti_remove_table_model (eti);
 	eti_remove_selection_model (eti);
 
-	if (eti->height_cache_idle_id)
+	if (eti->height_cache_idle_id) {
 		g_source_remove(eti->height_cache_idle_id);
+		eti->height_cache_idle_id = 0;
+	}
 
 	if (eti->height_cache)
 		g_free (eti->height_cache);
@@ -1116,6 +1260,18 @@ eti_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 
 		e_table_item_focus (eti, cursor_col != -1 ? cursor_col : 0, view_to_model_row(eti, GTK_VALUE_INT (*arg)), 0);
 		break;
+	case ARG_UNIFORM_ROW_HEIGHT:
+		if (eti->uniform_row_height != GTK_VALUE_BOOL (*arg)) {
+			eti->uniform_row_height = GTK_VALUE_BOOL (*arg);
+			if (GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED) {
+				free_height_cache(eti);
+				eti->needs_compute_height = 1;
+				e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (eti));
+				eti->needs_redraw = 1;
+				gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (eti));
+			}
+		}
+		break;
 	}
 	eti->needs_redraw = 1;
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM(eti));
@@ -1147,8 +1303,12 @@ eti_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 			       NULL);
 		GTK_VALUE_INT (*arg) = model_to_view_row(eti, row);
 		break;
+	case ARG_UNIFORM_ROW_HEIGHT:
+		GTK_VALUE_BOOL (*arg) = eti->uniform_row_height;
+		break;
 	default:
 		arg->type = GTK_TYPE_INVALID;
+		break;
 	}
 }
 
@@ -1170,7 +1330,7 @@ eti_init (GnomeCanvasItem *item)
 	eti->height_cache_idle_count   = 0;
 	
 	eti->length_threshold          = -1;
-	eti->renderers_can_change_size = 1;
+	eti->uniform_row_height        = FALSE;
 
 	eti->uses_source_model         = 0;
 	eti->source_model              = NULL;
@@ -1203,6 +1363,12 @@ eti_init (GnomeCanvasItem *item)
 	eti->maybe_in_drag             = 0;
 	eti->grabbed                   = 0;
 
+	eti->cursor_on_screen          = FALSE;
+ 	eti->cursor_x1                 = -1;
+	eti->cursor_y1                 = -1;
+	eti->cursor_x2                 = -1;
+	eti->cursor_y2                 = -1;
+
 	eti->rows                      = -1;
 
 	e_canvas_item_set_reflow_callback (GNOME_CANVAS_ITEM (eti), eti_reflow);
@@ -1213,6 +1379,12 @@ eti_init (GnomeCanvasItem *item)
 static const char gray50_bits[] = {
 	0x02, 0x01, };
 
+static void
+adjustment_changed (GtkAdjustment *adjustment, ETableItem *eti)
+{
+	eti_check_cursor_on_screen (eti);
+}
+
 static void
 eti_realize (GnomeCanvasItem *item)
 {
@@ -1243,6 +1415,18 @@ eti_realize (GnomeCanvasItem *item)
 	gdk_gc_set_stipple (eti->focus_gc, eti->stipple);
 	gdk_gc_set_fill (eti->focus_gc, GDK_OPAQUE_STIPPLED);
 
+	eti->hadjustment_change_id =
+		gtk_signal_connect(GTK_OBJECT(gtk_layout_get_hadjustment(GTK_LAYOUT(item->canvas))), "changed",
+				   GTK_SIGNAL_FUNC (adjustment_changed), eti);
+	eti->hadjustment_value_change_id =
+		gtk_signal_connect(GTK_OBJECT(gtk_layout_get_hadjustment(GTK_LAYOUT(item->canvas))), "value_changed",
+				   GTK_SIGNAL_FUNC (adjustment_changed), eti);
+	eti->vadjustment_change_id =
+		gtk_signal_connect(GTK_OBJECT(gtk_layout_get_vadjustment(GTK_LAYOUT(item->canvas))), "changed",
+				   GTK_SIGNAL_FUNC (adjustment_changed), eti);
+	eti->vadjustment_value_change_id =
+		gtk_signal_connect(GTK_OBJECT(gtk_layout_get_vadjustment(GTK_LAYOUT(item->canvas))), "value_changed",
+				   GTK_SIGNAL_FUNC (adjustment_changed), eti);
 
 	if (eti->cell_views == NULL)
 		eti_attach_cell_views (eti);
@@ -1262,9 +1446,9 @@ static void
 eti_unrealize (GnomeCanvasItem *item)
 {
 	ETableItem *eti = E_TABLE_ITEM (item);
-	
+
 	if (eti->grabbed) {
-		gtk_grab_remove (GTK_WIDGET (GNOME_CANVAS_ITEM (eti)->canvas));
+		gtk_grab_remove (GTK_WIDGET (item->canvas));
 		eti->grabbed = FALSE;
 	}
 
@@ -1279,11 +1463,20 @@ eti_unrealize (GnomeCanvasItem *item)
 	eti->focus_gc = NULL;
 	gdk_bitmap_unref (eti->stipple);
 	eti->stipple = NULL;
-	
+
 	eti_unrealize_cell_views (eti);
 
 	eti->height = 0;
-	
+
+	gtk_signal_disconnect(GTK_OBJECT(gtk_layout_get_hadjustment(GTK_LAYOUT(item->canvas))),
+			      eti->hadjustment_change_id);
+	gtk_signal_disconnect(GTK_OBJECT(gtk_layout_get_hadjustment(GTK_LAYOUT(item->canvas))),
+			      eti->hadjustment_value_change_id);
+	gtk_signal_disconnect(GTK_OBJECT(gtk_layout_get_vadjustment(GTK_LAYOUT(item->canvas))),
+			      eti->vadjustment_change_id);
+	gtk_signal_disconnect(GTK_OBJECT(gtk_layout_get_vadjustment(GTK_LAYOUT(item->canvas))),
+			      eti->vadjustment_value_change_id);
+
 	if (GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->unrealize)
                 (*GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->unrealize)(item);
 }
@@ -1296,7 +1489,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	ETableItem *eti = E_TABLE_ITEM (item);
 	const int rows = eti->rows;
 	const int cols = eti->cols;
-	int row, col, y1, y2;
+	int row, col;
 	int first_col, last_col, x_offset;
 	int first_row, last_row, y_offset, yd;
 	int x1, x2;
@@ -1329,7 +1522,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 		ETableCol *ecol = e_table_header_get_column (eti->header, col);
 
 		x2 = x1 + ecol->width;
-		
+
 		if (x1 > (x + width))
 			break;
 		if (x2 < x)
@@ -1350,28 +1543,43 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	/*
 	 * Compute row span.
 	 */
-	first_row = -1;
-	y_offset = 0;
-	y1 = y2 = floor (eti_base.y) + height_extra;
-	for (row = 0; row < rows; row++, y1 = y2){
+	if (eti->uniform_row_height) {
+		first_row = (y          - floor (eti_base.y) - height_extra) / (eti_row_height (eti, -1) + height_extra);
+		last_row  = (y + height - floor (eti_base.y)               ) / (eti_row_height (eti, -1) + height_extra) + 1;
+		if (first_row > last_row)
+			return;
+		y_offset = floor (eti_base.y) - y + height_extra + first_row * (eti_row_height (eti, -1) + height_extra);
+		if (first_row < 0)
+			first_row = 0;
+		if (last_row > eti->rows)
+			last_row = eti->rows;
+	} else {
+		int y1, y2;
 
-		y2 += ETI_ROW_HEIGHT (eti, row) + height_extra;
+		y_offset = 0;
+		first_row = -1;
 
-		if (y1 > y + height)
-			break;
+		y1 = y2 = floor (eti_base.y) + height_extra;
+		for (row = 0; row < rows; row++, y1 = y2){
 
-		if (y2 < y)
-			continue;
+			y2 += ETI_ROW_HEIGHT (eti, row) + height_extra;
 
-		if (first_row == -1){
-			y_offset = y1 - y;
-			first_row = row;
+			if (y1 > y + height)
+				break;
+
+			if (y2 < y)
+				continue;
+
+			if (first_row == -1){
+				y_offset = y1 - y;
+				first_row = row;
+			}
 		}
-	}
-	last_row = row;
+		last_row = row;
 
-	if (first_row == -1)
-		return;
+		if (first_row == -1)
+			return;
+	}
 
 	/*
 	 * Draw cells
@@ -1566,15 +1774,23 @@ find_cell (ETableItem *eti, double x, double y, int *view_col_res, int *view_row
 			break;
 	}
 
-	y1 = y2 = 0;
-	for (row = 0; row < rows - 1; row++, y1 = y2){
-		if (y < y1)
+	if (eti->uniform_row_height) {
+		if (y < height_extra)
 			return FALSE;
-		
-		y2 += ETI_ROW_HEIGHT (eti, row) + height_extra;
+		row = (y - height_extra) / (eti_row_height (eti, -1) + height_extra);
+		y1 = row * (eti_row_height (eti, -1) + height_extra) + height_extra;
+		if (row >= eti->rows)
+			return FALSE;
+	} else {
+		y1 = y2 = height_extra;
+		if (y < height_extra) 
+			return FALSE;
+		for (row = 0; row < rows - 1; row++, y1 = y2){
+			y2 += ETI_ROW_HEIGHT (eti, row) + height_extra;
 
-		if (y <= y2)
-			break;
+			if (y <= y2)
+				break;
+		}
 	}
 	*view_col_res = col;
 	if (x1_res)
@@ -1621,9 +1837,6 @@ static int
 _do_tooltip (ETableItem *eti)
 {
 	ECellView *ecell_view;
-	int x = 0, y = 0;
-	int i;
-	int height_extra = eti->horizontal_draw_grid ? 1 : 0;
 	gboolean free_color;
 	ETableCol *ecol;
 	gboolean selected;
@@ -1636,14 +1849,10 @@ _do_tooltip (ETableItem *eti)
 
 	ecell_view = eti->cell_views[eti->tooltip->col];
 
-	for (i = 0; i < eti->tooltip->col; i++)
-		x += eti->header->columns[i]->width;
-	eti->tooltip->x = x;
+	eti->tooltip->x = e_table_header_col_diff (eti->header, 0, eti->tooltip->col);
 
-	for (i = 0; i < eti->tooltip->row; i++)
-		y += (ETI_ROW_HEIGHT (eti, i) + height_extra);
-	eti->tooltip->y = y;
-	eti->tooltip->row_height = ETI_ROW_HEIGHT (eti, i);
+	eti->tooltip->y = e_table_item_row_diff (eti, 0, eti->tooltip->row);
+	eti->tooltip->row_height = ETI_ROW_HEIGHT (eti, eti->tooltip->row);
 
 	selected = e_selection_model_is_row_selected(E_SELECTION_MODEL (eti->selection), view_to_model_row(eti,eti->tooltip->row));
 
@@ -1845,6 +2054,11 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		gint cursor_row, cursor_col;
 
 		if (e->button.button == 1) {
+			if (eti->grabbed) {
+				gtk_grab_remove (GTK_WIDGET (item->canvas));
+				gnome_canvas_item_ungrab(item, e->button.time);
+			}
+			eti->grabbed = FALSE;
 			if (eti->maybe_in_drag) {
 				eti->maybe_in_drag = FALSE;
 				if (!eti->maybe_did_something)
@@ -1853,11 +2067,6 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			if (eti->in_drag) {
 				eti->in_drag = FALSE;
 			}
-			if (eti->grabbed) {
-				gtk_grab_remove (GTK_WIDGET (item->canvas));
-				gnome_canvas_item_ungrab(item, e->button.time);
-			}
-			eti->grabbed = FALSE;
 		}
 
 		if (eti->tooltip->timer) {
@@ -1912,8 +2121,6 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 #if 0
 		double x1, y1;
 #endif
-		GdkEventButton button;
-
 		if (e->button.button == 5 ||
 		    e->button.button == 4)
 			return FALSE;
@@ -1937,18 +2144,17 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				       "cursor_col", &model_col,
 				       NULL);
 
-			button.x -= e_table_header_col_diff (eti->header, 0, model_to_view_col (eti, model_col));
-			button.y -= e_table_item_row_diff (eti, 0, model_to_view_row (eti, model_row));
+			e->button.x -= e_table_header_col_diff (eti->header, 0, model_to_view_col (eti, model_col));
+			e->button.y -= e_table_item_row_diff (eti, 0, model_to_view_row (eti, model_row));
 
 #if 0
-			button = *(GdkEventButton *)e;
 			button.x = x1;
 			button.y = y1;
 #endif
 
 			if (model_row != -1 && model_col != -1) {
 				gtk_signal_emit (GTK_OBJECT (eti), eti_signals [DOUBLE_CLICK],
-						 model_row, model_col, &button);
+						 model_row, model_col, e);
 				d(g_print("Double click\n"));
 			}
 		}
@@ -2287,11 +2493,13 @@ eti_class_init (GtkObjectClass *object_class)
 				 GTK_ARG_READABLE, ARG_HEIGHT);
 	gtk_object_add_arg_type ("ETableItem::cursor_row", GTK_TYPE_INT,
 				 GTK_ARG_READWRITE, ARG_CURSOR_ROW);
+	gtk_object_add_arg_type ("ETableItem::uniform_row_height", GTK_TYPE_BOOL,
+				 GTK_ARG_READWRITE, ARG_UNIFORM_ROW_HEIGHT);
 
 	eti_signals [CURSOR_CHANGE] =
 		gtk_signal_new ("cursor_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, cursor_change),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -2299,7 +2507,7 @@ eti_class_init (GtkObjectClass *object_class)
 	eti_signals [CURSOR_ACTIVATED] =
 		gtk_signal_new ("cursor_activated",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, cursor_activated),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -2307,7 +2515,7 @@ eti_class_init (GtkObjectClass *object_class)
 	eti_signals [DOUBLE_CLICK] =
 		gtk_signal_new ("double_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, double_click),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2315,7 +2523,7 @@ eti_class_init (GtkObjectClass *object_class)
 	eti_signals [START_DRAG] =
 		gtk_signal_new ("start_drag",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, start_drag),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2323,7 +2531,7 @@ eti_class_init (GtkObjectClass *object_class)
 	eti_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, right_click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2331,7 +2539,7 @@ eti_class_init (GtkObjectClass *object_class)
 	eti_signals [CLICK] =
 		gtk_signal_new ("click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2339,12 +2547,12 @@ eti_class_init (GtkObjectClass *object_class)
 	eti_signals [KEY_PRESS] =
 		gtk_signal_new ("key_press",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableItemClass, key_press),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
-	gtk_object_class_add_signals (object_class, eti_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, eti_signals, LAST_SIGNAL);
 
 }
 
@@ -2448,9 +2656,9 @@ eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti
 
 	if (! e_table_model_has_change_pending (eti->table_model)) {
 		if (!eti->in_key_press) {
-			eti_request_region_show (eti, view_col, view_row, view_col, view_row, DOUBLE_CLICK_TIME + 10);
+			eti_maybe_show_cursor(eti, DOUBLE_CLICK_TIME + 10);
 		} else {
-			eti_request_region_show (eti, view_col, view_row, view_col, view_row, 0);
+			eti_maybe_show_cursor(eti, 0);
 		}
 	}
 
@@ -2480,6 +2688,14 @@ eti_cursor_activated (ESelectionModel *selection, int row, int col, ETableItem *
 		return;
 	}
 
+	if (! e_table_model_has_change_pending (eti->table_model)) {
+		if (!eti->in_key_press) {
+			eti_show_cursor(eti, DOUBLE_CLICK_TIME + 10);
+		} else {
+			eti_show_cursor(eti, 0);
+		}
+	}
+
 	if (eti_editing(eti))
 		e_table_item_leave_edit (eti);
 	gtk_signal_emit (GTK_OBJECT (eti), eti_signals [CURSOR_ACTIVATED],
