@@ -49,6 +49,11 @@ typedef struct {
 	int              selection_change_id;
 	int              cursor_change_id;
 	int              cursor_activated_id;
+
+	int              hadjustment_change_id;
+	int              hadjustment_value_change_id;
+	int              vadjustment_change_id;
+	int              vadjustment_value_change_id;
 	
 	GdkGC           *fill_gc;
 	GdkGC           *grid_gc;
@@ -59,7 +64,7 @@ typedef struct {
 	guint 		 horizontal_draw_grid:1;
 	guint 		 vertical_draw_grid:1;
 	guint 		 draw_focus:1;
-	guint 		 renderers_can_change_size:1;
+	guint 		 uniform_row_height:1;
 	guint 		 cell_views_realized:1;
 	      	    
 	guint 		 needs_redraw : 1;
@@ -76,6 +81,13 @@ typedef struct {
 
 	guint            maybe_did_something : 1;
 
+	guint            cursor_on_screen : 1;
+
+	int              cursor_x1;
+	int              cursor_y1;
+	int              cursor_x2;
+	int              cursor_y2;
+
 	int    		 drag_col;
 	int    		 drag_row;
 	int    		 drag_x;
@@ -89,6 +101,7 @@ typedef struct {
 	int              n_cells;
 
 	int             *height_cache;
+	int              uniform_row_height_cache;
 	int              height_cache_idle_id;
 	int              height_cache_idle_count;
 
