@@ -70,6 +70,7 @@ enum {
 	ARG_0,
 	ARG_LENGTH_THRESHOLD,
 	ARG_MODEL,
+	ARG_UNIFORM_ROW_HEIGHT,
 };
 
 static gint et_signals [LAST_SIGNAL] = { 0, };
@@ -219,6 +220,7 @@ e_table_init (GtkObject *object)
 	e_table->draw_focus                         = 1;
 	e_table->cursor_mode                        = E_CURSOR_SIMPLE;
 	e_table->length_threshold                   = 200;
+	e_table->uniform_row_height                 = FALSE;
 
 	e_table->need_rebuild                       = 0;
 	e_table->rebuild_idle_id                    = 0;
@@ -618,6 +620,7 @@ et_build_groups (ETable *et)
 			      "drawfocus", et->draw_focus,
 			      "cursor_mode", et->cursor_mode,
 			      "length_threshold", et->length_threshold,
+			      "uniform_row_height", et->uniform_row_height,
 			      "selection_model", et->selection,
 			      NULL);
 
@@ -1423,7 +1426,9 @@ et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_MODEL:
 		GTK_VALUE_OBJECT (*arg) = (GtkObject *) etable->model;
 		break;
-
+	case ARG_UNIFORM_ROW_HEIGHT:
+		GTK_VALUE_BOOL (*arg) = etable->uniform_row_height;
+		break;
 	default:
 		break;
 	}
@@ -1448,7 +1453,14 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 					       NULL);
 		}
 		break;
-
+	case ARG_UNIFORM_ROW_HEIGHT:
+		etable->uniform_row_height = GTK_VALUE_BOOL (*arg);
+		if (etable->group) {
+			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etable->group),
+					       "uniform_row_height", GTK_VALUE_BOOL (*arg),
+					       NULL);
+		}
+		break;
 	}
 }
 
@@ -2066,6 +2078,27 @@ scroll_off (ETable *et)
 	}
 }
 
+static void
+context_destroyed (gpointer data)
+{
+	ETable *et = data;
+	et->last_drop_x       = 0;
+	et->last_drop_y       = 0;
+	et->last_drop_time    = 0;
+	et->last_drop_context = NULL;
+	scroll_off (et);
+	gtk_object_unref (GTK_OBJECT (et));
+}
+
+static void
+context_connect (ETable *et, GdkDragContext *context)
+{
+	if (g_dataset_get_data (context, "e-table") == NULL) {
+		gtk_object_ref (GTK_OBJECT (et));
+		g_dataset_set_data_full (context, "e-table", et, context_destroyed);
+	}
+}
+
 static void
 et_drag_leave(GtkWidget *widget,
 	      GdkDragContext *context,
@@ -2098,6 +2131,7 @@ et_drag_motion(GtkWidget *widget,
 	et->last_drop_y = y;
 	et->last_drop_time = time;
 	et->last_drop_context = context;
+	context_connect (et, context);
 
 	ret_val = do_drag_motion (et,
 				  context,
@@ -2245,7 +2279,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [CURSOR_CHANGE] =
 		gtk_signal_new ("cursor_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, cursor_change),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -2253,7 +2287,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [CURSOR_ACTIVATED] =
 		gtk_signal_new ("cursor_activated",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, cursor_activated),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -2261,7 +2295,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [SELECTION_CHANGE] =
 		gtk_signal_new ("selection_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, selection_change),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -2269,7 +2303,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [DOUBLE_CLICK] =
 		gtk_signal_new ("double_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, double_click),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2277,7 +2311,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, right_click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2285,7 +2319,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [CLICK] =
 		gtk_signal_new ("click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2293,7 +2327,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [KEY_PRESS] =
 		gtk_signal_new ("key_press",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, key_press),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2301,7 +2335,7 @@ e_table_class_init (ETableClass *class)
 	et_signals [START_DRAG] =
 		gtk_signal_new ("start_drag",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, start_drag),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2309,7 +2343,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_BEGIN] =
 		gtk_signal_new ("table_drag_begin",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_begin),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3,
@@ -2319,7 +2353,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_END] =
 		gtk_signal_new ("table_drag_end",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_end),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3,
@@ -2329,7 +2363,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DATA_GET] =
 		gtk_signal_new ("table_drag_data_get",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_data_get),
 				e_marshal_NONE__INT_INT_POINTER_POINTER_UINT_UINT,
 				GTK_TYPE_NONE, 6,
@@ -2342,7 +2376,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DATA_DELETE] =
 		gtk_signal_new ("table_drag_data_delete",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_data_delete),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3,
@@ -2353,7 +2387,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_LEAVE] =
 		gtk_signal_new ("table_drag_leave",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_leave),
 				e_marshal_NONE__INT_INT_POINTER_UINT,
 				GTK_TYPE_NONE, 4,
@@ -2364,7 +2398,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_MOTION] =
 		gtk_signal_new ("table_drag_motion",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_motion),
 				e_marshal_BOOL__INT_INT_POINTER_INT_INT_UINT,
 				GTK_TYPE_BOOL, 6,
@@ -2377,7 +2411,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DROP] =
 		gtk_signal_new ("table_drag_drop",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_drop),
 				e_marshal_BOOL__INT_INT_POINTER_INT_INT_UINT,
 				GTK_TYPE_BOOL, 6,
@@ -2390,7 +2424,7 @@ e_table_class_init (ETableClass *class)
 	et_signals[TABLE_DRAG_DATA_RECEIVED] =
 		gtk_signal_new ("table_drag_data_received",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, table_drag_data_received),
 				e_marshal_NONE__INT_INT_POINTER_INT_INT_POINTER_UINT_UINT,
 				GTK_TYPE_NONE, 8,
@@ -2403,20 +2437,22 @@ e_table_class_init (ETableClass *class)
 				GTK_TYPE_UINT,
 				GTK_TYPE_UINT);
 
-	gtk_object_class_add_signals (object_class, et_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, et_signals, LAST_SIGNAL);
 
 	class->set_scroll_adjustments = set_scroll_adjustments;
 
 	widget_class->set_scroll_adjustments_signal =
 		gtk_signal_new ("set_scroll_adjustments",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableClass, set_scroll_adjustments),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
 
 	gtk_object_add_arg_type ("ETable::length_threshold", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_LENGTH_THRESHOLD);
+	gtk_object_add_arg_type ("ETable::uniform_row_height", GTK_TYPE_BOOL,
+				 GTK_ARG_READWRITE, ARG_UNIFORM_ROW_HEIGHT);
 	gtk_object_add_arg_type ("ETable::model", E_TABLE_MODEL_TYPE,
 				 GTK_ARG_READABLE, ARG_MODEL);
 }
