@@ -74,7 +74,8 @@ enum {
 	ARG_HORIZONTAL_DRAW_GRID,
 	ARG_VERTICAL_DRAW_GRID,
 	ARG_DRAW_FOCUS,
-	ARG_ETTA
+	ARG_ETTA,
+	ARG_UNIFORM_ROW_HEIGHT,
 };
 
 struct ETreePriv {
@@ -124,6 +125,8 @@ struct ETreePriv {
 
 	guint do_drag : 1;
 
+	guint uniform_row_height : 1;
+
 	ECursorMode cursor_mode;
 
 	int drop_row;
@@ -295,6 +298,7 @@ e_tree_init (GtkObject *object)
 	e_tree->priv->draw_focus                         = 1;
 	e_tree->priv->cursor_mode                        = E_CURSOR_SIMPLE;
 	e_tree->priv->length_threshold                   = 200;
+	e_tree->priv->uniform_row_height                 = FALSE;
 
 	e_tree->priv->row_selection_active               = FALSE;
 	e_tree->priv->horizontal_scrolling               = FALSE;
@@ -652,6 +656,7 @@ et_build_item (ETree *et)
 					 "drawfocus", et->priv->draw_focus,
 					 "cursor_mode", et->priv->cursor_mode,
 					 "length_threshold", et->priv->length_threshold,
+					 "uniform_row_height", et->priv->uniform_row_height,
 					 NULL);
 
 	gtk_signal_connect (GTK_OBJECT (et->priv->item), "cursor_change",
@@ -1284,6 +1289,9 @@ et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 			GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (etree->priv->etta);
 		}
 		break;
+	case ARG_UNIFORM_ROW_HEIGHT:
+		GTK_VALUE_BOOL (*arg) = etree->priv->uniform_row_height;
+		break;
 
 	default:
 		break;
@@ -1336,6 +1344,15 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 					       NULL);
 		}
 		break;
+
+	case ARG_UNIFORM_ROW_HEIGHT:
+		etree->priv->uniform_row_height = GTK_VALUE_BOOL (*arg);
+		if (etree->priv->item) {
+			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etree->priv->item),
+					       "uniform_row_height", GTK_VALUE_BOOL (*arg),
+					       NULL);
+		}
+		break;
 	}
 }
 
@@ -2193,6 +2210,28 @@ hover_off (ETree *et)
 	}
 }
 
+static void
+context_destroyed (gpointer data)
+{
+	ETree *et = data;
+	et->priv->last_drop_x       = 0;
+	et->priv->last_drop_y       = 0;
+	et->priv->last_drop_time    = 0;
+	et->priv->last_drop_context = NULL;
+	scroll_off (et);
+	hover_off (et);
+	gtk_object_unref (GTK_OBJECT (et));
+}
+
+static void
+context_connect (ETree *et, GdkDragContext *context)
+{
+	if (g_dataset_get_data (context, "e-tree") == NULL) {
+		gtk_object_ref (GTK_OBJECT (et));
+		g_dataset_set_data_full (context, "e-tree", et, context_destroyed);
+	}
+}
+
 static void
 et_drag_leave(GtkWidget *widget,
 	      GdkDragContext *context,
@@ -2227,6 +2266,7 @@ et_drag_motion(GtkWidget *widget,
 	et->priv->last_drop_y = y;
 	et->priv->last_drop_time = time;
 	et->priv->last_drop_context = context;
+	context_connect (et, context);
 
 	if (et->priv->hover_idle_id != 0) {
 		if (abs (et->priv->hover_x - x) > 3 ||
@@ -2396,7 +2436,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [CURSOR_CHANGE] =
 		gtk_signal_new ("cursor_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, cursor_change),
 				gtk_marshal_NONE__INT_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_INT, GTK_TYPE_POINTER);
@@ -2404,7 +2444,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [CURSOR_ACTIVATED] =
 		gtk_signal_new ("cursor_activated",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, cursor_activated),
 				gtk_marshal_NONE__INT_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_INT, GTK_TYPE_POINTER);
@@ -2412,7 +2452,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [SELECTION_CHANGE] =
 		gtk_signal_new ("selection_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, selection_change),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -2420,7 +2460,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [DOUBLE_CLICK] =
 		gtk_signal_new ("double_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, double_click),
 				e_marshal_NONE__INT_POINTER_INT_POINTER,
 				GTK_TYPE_NONE, 4, GTK_TYPE_INT, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2428,7 +2468,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, right_click),
 				e_marshal_INT__INT_POINTER_INT_POINTER,
 				GTK_TYPE_INT, 4, GTK_TYPE_INT, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2436,7 +2476,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [CLICK] =
 		gtk_signal_new ("click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, click),
 				e_marshal_INT__INT_POINTER_INT_POINTER,
 				GTK_TYPE_INT, 4, GTK_TYPE_INT, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2444,7 +2484,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [KEY_PRESS] =
 		gtk_signal_new ("key_press",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, key_press),
 				e_marshal_INT__INT_POINTER_INT_POINTER,
 				GTK_TYPE_INT, 4, GTK_TYPE_INT, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2452,7 +2492,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals [START_DRAG] =
 		gtk_signal_new ("start_drag",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, start_drag),
 				e_marshal_NONE__INT_POINTER_INT_POINTER,
 				GTK_TYPE_NONE, 4, GTK_TYPE_INT, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -2460,7 +2500,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_BEGIN] =
 		gtk_signal_new ("tree_drag_begin",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_begin),
 				e_marshal_NONE__INT_POINTER_INT_POINTER,
 				GTK_TYPE_NONE, 4,
@@ -2471,7 +2511,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_END] =
 		gtk_signal_new ("tree_drag_end",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_end),
 				e_marshal_NONE__INT_POINTER_INT_POINTER,
 				GTK_TYPE_NONE, 4,
@@ -2482,7 +2522,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_DATA_GET] =
 		gtk_signal_new ("tree_drag_data_get",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_data_get),
 				e_marshal_NONE__INT_POINTER_INT_POINTER_POINTER_UINT_UINT,
 				GTK_TYPE_NONE, 7,
@@ -2496,7 +2536,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_DATA_DELETE] =
 		gtk_signal_new ("tree_drag_data_delete",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_data_delete),
 				e_marshal_NONE__INT_POINTER_INT_POINTER,
 				GTK_TYPE_NONE, 4,
@@ -2508,7 +2548,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_LEAVE] =
 		gtk_signal_new ("tree_drag_leave",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_leave),
 				e_marshal_NONE__INT_POINTER_INT_POINTER_UINT,
 				GTK_TYPE_NONE, 5,
@@ -2520,7 +2560,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_MOTION] =
 		gtk_signal_new ("tree_drag_motion",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_motion),
 				e_marshal_BOOL__INT_POINTER_INT_POINTER_INT_INT_UINT,
 				GTK_TYPE_BOOL, 7,
@@ -2534,7 +2574,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_DROP] =
 		gtk_signal_new ("tree_drag_drop",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_drop),
 				e_marshal_BOOL__INT_POINTER_INT_POINTER_INT_INT_UINT,
 				GTK_TYPE_BOOL, 7,
@@ -2548,7 +2588,7 @@ e_tree_class_init (ETreeClass *class)
 	et_signals[TREE_DRAG_DATA_RECEIVED] =
 		gtk_signal_new ("tree_drag_data_received",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, tree_drag_data_received),
 				e_marshal_NONE__INT_POINTER_INT_POINTER_INT_INT_POINTER_UINT_UINT,
 				GTK_TYPE_NONE, 9,
@@ -2562,14 +2602,14 @@ e_tree_class_init (ETreeClass *class)
 				GTK_TYPE_UINT,
 				GTK_TYPE_UINT);
 
-	gtk_object_class_add_signals (object_class, et_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, et_signals, LAST_SIGNAL);
 
 	class->set_scroll_adjustments = set_scroll_adjustments;
 
 	widget_class->set_scroll_adjustments_signal =
 		gtk_signal_new ("set_scroll_adjustments",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETreeClass, set_scroll_adjustments),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
@@ -2584,6 +2624,8 @@ e_tree_class_init (ETreeClass *class)
 				 GTK_ARG_WRITABLE, ARG_DRAW_FOCUS);
 	gtk_object_add_arg_type ("ETree::ETreeTableAdapter", GTK_TYPE_OBJECT,
 				 GTK_ARG_READABLE, ARG_ETTA);
+	gtk_object_add_arg_type ("ETree::uniform_row_height", GTK_TYPE_BOOL,
+				 GTK_ARG_READWRITE, ARG_UNIFORM_ROW_HEIGHT);
 }
 
 E_MAKE_TYPE(e_tree, "ETree", ETree, e_tree_class_init, e_tree_init, PARENT_TYPE);
