@@ -225,7 +225,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_NONEMPTY] =
 		gtk_signal_new ("nonempty",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, nonempty),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -233,7 +233,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_ADDED] =
 		gtk_signal_new ("added",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, added),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -241,7 +241,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_FULL] =
 		gtk_signal_new ("full",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, full),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -249,7 +249,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_BROWSE] =
 		gtk_signal_new ("browse",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, browse),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1,
@@ -258,7 +258,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_UNBROWSE] =
 		gtk_signal_new ("unbrowse",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, unbrowse),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -266,13 +266,13 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_ACTIVATE] =
 		gtk_signal_new ("activate",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, activate),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_POINTER);
 
-	gtk_object_class_add_signals (object_class, e_completion_view_signals, E_COMPLETION_VIEW_LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_completion_view_signals, E_COMPLETION_VIEW_LAST_SIGNAL);
 
 	object_class->destroy = e_completion_view_destroy;
 
@@ -452,16 +452,28 @@ e_completion_view_key_press_handler (GtkWidget *w, GdkEventKey *key_event, gpoin
 {
 	ECompletionView *cv = E_COMPLETION_VIEW (user_data);
 	gint dir = 0;
-	gboolean key_handled = TRUE;
+	gboolean key_handled = TRUE, complete_key = FALSE, uncomplete_key = FALSE;
+
+	/* FIXME: This is totally lame.
+	   The ECompletionView should be able to specify multiple completion/uncompletion keys, or just
+	   have sensible defaults. */
+	
+	if ((cv->complete_key && key_event->keyval == cv->complete_key)
+	    || ((key_event->keyval == GDK_n || key_event->keyval == GDK_N) && (key_event->state & GDK_CONTROL_MASK)))
+		complete_key = TRUE;
+
+	if ((cv->uncomplete_key && key_event->keyval == cv->uncomplete_key)
+	    || ((key_event->keyval == GDK_p || key_event->keyval == GDK_P) && (key_event->state & GDK_CONTROL_MASK)))
+		uncomplete_key = TRUE;
 	
 	/* Start up a completion.*/
-	if (cv->complete_key && key_event->keyval == cv->complete_key && !cv->editable) {
+	if (complete_key && !cv->editable) {
 		gtk_signal_emit (GTK_OBJECT (cv), e_completion_view_signals[E_COMPLETION_VIEW_BROWSE], NULL);
 		goto stop_emission;
 	}
 
 	/* Stop our completion. */
-	if (cv->uncomplete_key && key_event->keyval == cv->uncomplete_key && cv->editable && cv->selection < 0) {
+	if (uncomplete_key && cv->editable && cv->selection < 0) {
 		e_completion_view_set_cursor_row (cv, -1);
 		gtk_signal_emit (GTK_OBJECT (cv), e_completion_view_signals[E_COMPLETION_VIEW_UNBROWSE]);
 		goto stop_emission;
@@ -471,11 +483,24 @@ e_completion_view_key_press_handler (GtkWidget *w, GdkEventKey *key_event, gpoin
 		return FALSE;
 
 	switch (key_event->keyval) {
+
+	case GDK_n:
+	case GDK_N:
+		/* We (heart) emacs: treat ctrl-n as down */
+		if (! (key_event->state & GDK_CONTROL_MASK))
+			return FALSE;
+
 	case GDK_Down:
 	case GDK_KP_Down:
 		dir = 1;
 		break;
 
+	case GDK_p:
+	case GDK_P:
+		/* Treat ctrl-p as up */
+		if (! (key_event->state & GDK_CONTROL_MASK))
+			return FALSE;
+		
 	case GDK_Up:
 	case GDK_KP_Up:
 		dir = -1;
