@@ -39,6 +39,7 @@
 #include <gal/e-text/e-text.h>
 #include <gal/widgets/e-canvas-utils.h>
 #include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include "gal/util/e-util.h"
 
 /* These are the offsets of the icons & text in both views. Note that the
    shadow around icons is drawn in the space between items (as is the
@@ -50,8 +51,8 @@
 #define E_ICON_BAR_LARGE_ICON_TEXT_Y	(E_ICON_BAR_LARGE_ICON_HEIGHT + 4)
 
 #define E_ICON_BAR_SMALL_ICON_SPACING	4	/* Spacing between items. */
-#define E_ICON_BAR_SMALL_ICON_WIDTH	24
-#define E_ICON_BAR_SMALL_ICON_HEIGHT	24
+#define E_ICON_BAR_SMALL_ICON_WIDTH	16
+#define E_ICON_BAR_SMALL_ICON_HEIGHT	16
 #define E_ICON_BAR_SMALL_ICON_X		4
 #define E_ICON_BAR_SMALL_ICON_TEXT_X	(E_ICON_BAR_SMALL_ICON_WIDTH + 6)
 
@@ -185,7 +186,7 @@ e_icon_bar_class_init (EIconBarClass *class)
 	e_icon_bar_signals[ITEM_SELECTED] =
 		gtk_signal_new ("item_selected",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EIconBarClass,
 						   selected_item),
 				gtk_marshal_NONE__POINTER_INT,
@@ -194,14 +195,14 @@ e_icon_bar_class_init (EIconBarClass *class)
 	e_icon_bar_signals[ITEM_DRAGGED] =
 		gtk_signal_new ("item_dragged",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EIconBarClass,
 						   dragged_item),
 				gtk_marshal_NONE__POINTER_INT,
 				GTK_TYPE_NONE, 2, GTK_TYPE_GDK_EVENT,
 				GTK_TYPE_INT);
 
-	gtk_object_class_add_signals (object_class, e_icon_bar_signals,
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_icon_bar_signals,
 				      LAST_SIGNAL);
 
 	/* Method override */
@@ -677,7 +678,8 @@ e_icon_bar_add_item (EIconBar	    *icon_bar,
 			    icon_bar);
 
 	item.pixbuf = image;
-	gdk_pixbuf_ref (image);
+	if (image)
+		gdk_pixbuf_ref (image);
 
 	flattened = flatten_alpha (image,
 				   rgb_from_gdk_color (&style->bg [GTK_STATE_NORMAL]));
