@@ -9,6 +9,7 @@
  */
 #include <config.h>
 #include <gtk/gtksignal.h>
+#include "gal/util/e-util.h"
 #include "e-categories-master-list.h"
 
 #define ECML_CLASS(e) ((ECategoriesMasterListClass *)((GtkObject *)e)->klass)
@@ -168,12 +169,12 @@ e_categories_master_list_class_init (GtkObjectClass *object_class)
 	signals [CHANGED] =
 		gtk_signal_new ("changed",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECategoriesMasterListClass, changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, signals, LAST_SIGNAL);
 
 	klass->count   = NULL;     
 	klass->nth     = NULL;         
