@@ -901,8 +901,12 @@ translate_encoding (const gchar *encoding)
 	g_strdown (e);
 
 	finalenc = g_hash_table_lookup (eh, e);
+	if (finalenc == NULL) {
+		finalenc = g_strdup(encoding);
+		g_hash_table_insert(eh, g_strdup(e), finalenc);
+	}
 
-	return finalenc ? finalenc : encoding;
+	return finalenc;
 }
 
 static const gchar *
