@@ -30,6 +30,7 @@
 #include "e-canvas.h"
 #include "gal/e-text/e-text.h"
 #include "gal/util/e-util.h"
+#include "gal/widgets/e-unicode.h"
 #include <gtk/gtksignal.h>
 #include "e-selection-model-simple.h"
 
@@ -347,19 +348,22 @@ set_empty(EReflow *reflow)
 	if (reflow->count == 0) {
 		if (reflow->empty_text) {
 			if (reflow->empty_message) {
+				char *empty_message = e_utf8_to_gtk_string (GTK_WIDGET (GNOME_CANVAS_ITEM (reflow)->canvas), reflow->empty_message);
 				gnome_canvas_item_set(reflow->empty_text,
 						      "width", reflow->minimum_width,
-						      "text", reflow->empty_message,
+						      "text", empty_message,
 						      NULL);
 				e_canvas_item_move_absolute(reflow->empty_text,
 							    reflow->minimum_width / 2,
 							    0);
+				g_free (empty_message);
 			} else {
 				gtk_object_destroy(GTK_OBJECT(reflow->empty_text));
 				reflow->empty_text = NULL;
 			}
 		} else {
-			if (reflow->empty_message)
+			if (reflow->empty_message) {
+				char *empty_message = e_utf8_to_gtk_string (GTK_WIDGET (GNOME_CANVAS_ITEM (reflow)->canvas), reflow->empty_message);
 				reflow->empty_text =
 					gnome_canvas_item_new(GNOME_CANVAS_GROUP(reflow),
 							      e_text_get_type(),
@@ -370,12 +374,14 @@ set_empty(EReflow *reflow)
 							      "font_gdk", GTK_WIDGET(GNOME_CANVAS_ITEM(reflow)->canvas)->style->font,
 							      "fill_color", "black",
 							      "justification", GTK_JUSTIFY_CENTER,
-							      "text", reflow->empty_message,
+							      "text", empty_message,
 							      "draw_background", FALSE,
 							      NULL);
-			e_canvas_item_move_absolute(reflow->empty_text,
-						    reflow->minimum_width / 2,
-						    0);
+				g_free (empty_message);
+				e_canvas_item_move_absolute(reflow->empty_text,
+							    reflow->minimum_width / 2,
+							    0);
+			}
 		}
 	} else {
 		if (reflow->empty_text) {
@@ -1174,12 +1180,12 @@ e_reflow_class_init (EReflowClass *klass)
 	signals [SELECTION_EVENT] =
 		gtk_signal_new ("selection_event",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EReflowClass, selection_event),
 				e_marshal_INT__OBJECT_POINTER,
 				GTK_TYPE_INT, 2, GTK_TYPE_OBJECT, GTK_TYPE_GDK_EVENT);
 
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, signals, LAST_SIGNAL);
 
 	object_class->set_arg  = e_reflow_set_arg;
 	object_class->get_arg  = e_reflow_get_arg;
