@@ -22,6 +22,7 @@
 #include <libgnomeui/gnome-preferences.h>
 #include <libgnomeui/gnome-stock.h>
 #include "gtk-combo-stack.h"
+#include "gal/util/e-util.h"
 
 static GtkObjectClass *gtk_combo_stack_parent_class;
 
@@ -60,12 +61,12 @@ gtk_combo_stack_class_init (GtkObjectClass *object_class)
 	gtk_combo_stack_signals [POP] = gtk_signal_new (
 		"pop",
 		GTK_RUN_LAST,
-		object_class->type,
+		E_OBJECT_CLASS_TYPE (object_class),
 		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pop_down_done),
 		gtk_marshal_NONE__INT,
 		GTK_TYPE_NONE, 1, GTK_TYPE_INT);
 	
-	gtk_object_class_add_signals (object_class, gtk_combo_stack_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, gtk_combo_stack_signals, LAST_SIGNAL);
 }
 
 static void
